/*****************************************************************************
 * opdb.c - OpServices NEB module
 *
 * Copyright (c) 2008 OpServices (http://www.opservices.com.br)
 *
 * Instructions:
 *
 * Compile with the following command:
 *     gcc -D_GNU_SOURCE -Wall -shared -o opdb.o opdb.c -L/usr/lib/mysql -lmysqlclient -DNSCORE
 *****************************************************************************/

#include <sys/types.h>
#include <regex.h>

/* include (minimum required) event broker header files */
#include "../include/nebmodules.h"
#include "../include/nebcallbacks.h"

/* include other event broker header files that we need for our work */
#include "../include/nebstructs.h"
#include "../include/broker.h"

/* include some Nagios stuff as well */
#include "../include/config.h"
#include "../include/common.h"
#include "../include/nagios.h"
#include "../include/comments.h"
#include "/usr/include/mysql/mysql.h"

#define DBNAME "opmon"
#define DBPERFNAME "opperf"
#define DBOPCFGNAME "opcfg"
#define DBCONF "/usr/local/nagios/etc/db.php"
#define FAILED_QUERIES_FILE "/usr/local/nagios/var/opmon.sql"
#define PERF_FAILED_QUERIES_FILE "/usr/local/nagios/var/opperf.sql"

/* External variables */
extern host *host_list;
extern service *service_list;

/* specify event broker API version (required) */
NEB_API_VERSION(CURRENT_NEB_API_VERSION);
void *opdb_module_handle=NULL;

struct service_node *service_root = NULL;
struct host_node *host_root = NULL;
char DBHOST[256];
char DBUSER[256];
char DBPASS[256];
char DBPORT[256];
MYSQL *dbHandle;

/* linked list structures */
struct service_node {
	int id;
	char *host_name;
	char *service_description;
	struct service_node *next;
};

struct host_node {
	int id;
	int template_id;
	char *host_name;
	struct host_node *next;
};

struct metric {
	char *metric;
	char *value;
	char *unit;
	char *warning;
	char *critical;
	char *value_min;
	char *value_max;
};

/* Callback functions */
int opdb_handle_notification(int event_type, void *data);
int opdb_handle_state_change(int event_type, void *data);
int opdb_handle_comment(int event_type, void *data);
int opdb_handle_host_check(int event_type, void *data);
int opdb_handle_service_check(int event_type, void *data);
int opdb_handle_downtime(int event_type, void *data);
int opdb_insert_initial_state(int event_type, void *data);
int opdb_handle_flapping_data(int event_type, void *data);
int opdb_handle_external_command(int event_type, void *data);

/* aux functions */
int opdb_find_host(char *host_name);
int opdb_find_service(char *hostname, char *service_description);
int opdb_add_service_node(struct service_node *lf);
int opdb_add_host_node(struct host_node *lf);
int opdb_populate_service_list();
int opdb_populate_host_list();
void timestamp_to_longdate(char *longdate, time_t timestamp);
void remove_apostrofe(char *string);
int write_to_db(char *input, unsigned long data_type, time_t *timestamp);
void parse_db_conf();
int db_connect();
int db_disconnect();
void write_failed_query(char *sql, char *file);
int opdb_process_perfdata(void *data);
int opdb_store_perfdata(void *data, struct metric *m);
MYSQL_RES * db_query_exec(char *sql, char *db);
char * replace_backslashes(char *s);
int get_template_id(int template_id);
void print_service_ll();
void print_host_ll();

/*
 * this function gets called when the 
 * module is loaded by the event broker 
 */
int nebmodule_init(int flags, char *args, nebmodule *handle)
{

	char *temp_buffer=NULL;
	time_t current_time;
	int ret;
	
	/* parse database conf */
	parse_db_conf();
	/* database connect */
	db_connect();
	/* save our handle */
	opdb_module_handle=handle;
	
	/* created ids linked lists */
	ret = opdb_populate_host_list();
	if (ret != OK) {
		write_to_all_logs("Unable to create services linked lists",NSLOG_INFO_MESSAGE);
		return ERROR;
	}

	ret = opdb_populate_service_list();
	if (ret != OK) { 
		write_to_all_logs("Unable to create hosts linked lists",NSLOG_INFO_MESSAGE);
		return ERROR;
	}

	//print_host_ll();
	//print_service_ll();

	/*
	 * set some info - this is completely optional,
	 * as Nagios doesn't do anything with this data 
	 */
	neb_set_module_info(opdb_module_handle,NEBMODULE_MODINFO_TITLE,"OpDB");
	neb_set_module_info(opdb_module_handle,NEBMODULE_MODINFO_AUTHOR,"OpServices");
	neb_set_module_info(opdb_module_handle,
			NEBMODULE_MODINFO_TITLE,"Copyright (c) 2008 - OpServices V1.1a");
	neb_set_module_info(opdb_module_handle,NEBMODULE_MODINFO_VERSION,"1.1a");
	neb_set_module_info(opdb_module_handle,NEBMODULE_MODINFO_LICENSE,"Comercial");
	neb_set_module_info(opdb_module_handle,
			NEBMODULE_MODINFO_DESC,"OpServices NEB module to dump data do DB.");
	
	/* log module info to the Nagios log file */
	write_to_all_logs("OpDB: Copyright (c) 2008 - OpServices (info@opservices.com.br) V1.1a",
			NSLOG_INFO_MESSAGE);
	
	/* log a message to the Nagios log file */
	asprintf(&temp_buffer,"OpDB using conf <%s>", DBCONF);
	write_to_all_logs(temp_buffer,NSLOG_INFO_MESSAGE);
	my_free(temp_buffer);
	asprintf(&temp_buffer,"OpDB: DBHost: <%s> DBPort: <%s> DBUser: <%s> DBPass: <%s>",
			DBHOST, DBPORT, DBUSER,DBPASS);
	
	write_to_all_logs(temp_buffer,NSLOG_INFO_MESSAGE);
	my_free(temp_buffer);
	
	time(&current_time);
	
	/* register to be notified of certain events... */
	neb_register_callback(NEBCALLBACK_CONTACT_NOTIFICATION_METHOD_DATA,opdb_module_handle,0,opdb_handle_notification);
	neb_register_callback(NEBCALLBACK_STATE_CHANGE_DATA,opdb_module_handle,0,opdb_handle_state_change);
	neb_register_callback(NEBCALLBACK_COMMENT_DATA,opdb_module_handle,0,opdb_handle_comment);
	neb_register_callback(NEBCALLBACK_HOST_CHECK_DATA,opdb_module_handle,0,opdb_handle_host_check);
	neb_register_callback(NEBCALLBACK_SERVICE_CHECK_DATA,opdb_module_handle,0,opdb_handle_service_check);
	neb_register_callback(NEBCALLBACK_DOWNTIME_DATA,opdb_module_handle,0,opdb_handle_downtime);
	neb_register_callback(NEBCALLBACK_PROCESS_DATA,opdb_module_handle,0,opdb_insert_initial_state);
	neb_register_callback(NEBCALLBACK_PROCESS_DATA,opdb_module_handle,0,opdb_handle_flapping_data);
	neb_register_callback(NEBCALLBACK_PROCESS_DATA,opdb_module_handle,0,opdb_handle_external_command);
	
	return OK;
}


/* 
 * this function gets called when the module
 * is unloaded by the event broker 
 */
int nebmodule_deinit(int flags, int reason)
{
	char *temp_buffer=NULL;
	
	/* database disconnect */
	db_disconnect();
	
	/* deregister for all events we previously registered for... */
	neb_deregister_callback(NEBCALLBACK_CONTACT_NOTIFICATION_METHOD_DATA,
			opdb_handle_notification);
	neb_deregister_callback(NEBCALLBACK_STATE_CHANGE_DATA,opdb_handle_state_change);
	neb_deregister_callback(NEBCALLBACK_COMMENT_DATA,opdb_handle_comment);
	neb_deregister_callback(NEBCALLBACK_HOST_CHECK_DATA,opdb_handle_host_check);
	neb_deregister_callback(NEBCALLBACK_SERVICE_CHECK_DATA,opdb_handle_service_check);
	neb_deregister_callback(NEBCALLBACK_DOWNTIME_DATA,opdb_handle_downtime);
	neb_deregister_callback(NEBCALLBACK_PROCESS_DATA,opdb_insert_initial_state);
	neb_deregister_callback(NEBCALLBACK_PROCESS_DATA,opdb_handle_flapping_data);
	neb_deregister_callback(NEBCALLBACK_PROCESS_DATA,opdb_handle_external_command);
	
	/* log a message to the Nagios log file */
	asprintf(&temp_buffer,"opdb: Tks for using OpServices broker\n");
	write_to_all_logs(temp_buffer,NSLOG_INFO_MESSAGE);
	my_free(temp_buffer);

	return OK;
}

/*
 * handle sent notification
 */
int opdb_handle_notification(int event_type, void *data)
{

	nebstruct_contact_notification_method_data *op_broker_data=NULL;
	char *sql=NULL;
	char longdate[32];
	char start_time[32];
	char end_time[32];
	int host_id;
	int service_id;
	MYSQL_RES *response = NULL;
	
	op_broker_data=(nebstruct_contact_notification_method_data *)data;
	
	if (op_broker_data->type==NEBTYPE_CONTACTNOTIFICATIONMETHOD_END) {
		
		timestamp_to_longdate(longdate, op_broker_data->timestamp.tv_sec);
		timestamp_to_longdate(start_time, op_broker_data->start_time.tv_sec);
		timestamp_to_longdate(end_time, op_broker_data->end_time.tv_sec);
		remove_apostrofe(op_broker_data->command_args);
		remove_apostrofe(op_broker_data->output);
		remove_apostrofe(op_broker_data->ack_data);
		
		host_id = opdb_find_host(op_broker_data->host_name);
		//printf("opdb_handle_notification Trying to find %s\n",op_broker_data->host_name);
		if (host_id == ERROR)
			return ERROR;
		
		/* Checks to see which notification type, SERVICE or HOST */
		if(op_broker_data->notification_type==SERVICE_NOTIFICATION) {
			service_id = opdb_find_service(op_broker_data->host_name,
				 op_broker_data->service_description);

			if (service_id == ERROR)
				return ERROR;


			asprintf(&sql,"INSERT INTO service_notification VALUES ('%s','%d','%d','%s','%s','%s',%d,%d,'%s','%s','%s',%d,'%s','%s')", longdate, host_id, service_id,op_broker_data->contact_name, op_broker_data->command_name, op_broker_data->command_args, op_broker_data->reason_type, op_broker_data->state, op_broker_data->output, op_broker_data->ack_author, op_broker_data->ack_data, op_broker_data->escalated, start_time, end_time);
			
		} else {
			asprintf(&sql,"INSERT INTO host_notification VALUES ('%s','%d','%s','%s','%s',%d,%d,'%s','%s','%s',%d,'%s','%s')", longdate, host_id, op_broker_data->contact_name, op_broker_data->command_name, op_broker_data->command_args, op_broker_data->reason_type, op_broker_data->state, op_broker_data->output, op_broker_data->ack_author, op_broker_data->ack_data, op_broker_data->escalated, start_time, end_time);
		}
		response = db_query_exec(sql,DBNAME);
		my_free(sql);
		mysql_free_result(response);
		
	}
	
	return OK;
}

/*
 * handle user's external command
 */
int opdb_handle_external_command(int event_type, void *data)
{
	nebstruct_external_command_data *op_broker_data=NULL;
	char *sql=NULL;
	char longdate[32];
	char entry_time[32];
	MYSQL_RES *response = NULL;
	
	op_broker_data=(nebstruct_external_command_data *)data;	
	if (op_broker_data->type==NEBTYPE_EXTERNALCOMMAND_START) {
		timestamp_to_longdate(longdate, op_broker_data->timestamp.tv_sec);
		timestamp_to_longdate(entry_time, op_broker_data->entry_time);
		remove_apostrofe(op_broker_data->command_string);
		remove_apostrofe(op_broker_data->command_args);
		asprintf(&sql,"INSERT INTO external_command VALUES ('%s','%d','%s','%s','%s')", longdate, op_broker_data->command_type, entry_time, op_broker_data->command_string,op_broker_data->command_args);
		response = db_query_exec(sql,DBNAME);
		my_free(sql);
		mysql_free_result(response);
	}

	return OK;

}

/*
 * handle hosts and services flapping states
 */
int opdb_handle_flapping_data(int event_type, void *data)
{

	nebstruct_flapping_data *op_broker_data=NULL;
	char *sql=NULL;
	char longdate[32];
	int host_id;
	int service_id;
	MYSQL_RES *response = NULL;
	
	op_broker_data=(nebstruct_flapping_data *)data;
	

	timestamp_to_longdate(longdate, op_broker_data->timestamp.tv_sec);
	
	if (op_broker_data->flapping_type==SERVICE_FLAPPING) {
	
		host_id = opdb_find_host(op_broker_data->host_name);
		//printf("opdb_handle_flapping_data Trying to find %s\n",op_broker_data->host_name);
		if (host_id == ERROR)
			return ERROR;
		
		service_id = opdb_find_service(op_broker_data->host_name,
			op_broker_data->service_description);

		if (service_id == ERROR)
			return ERROR;

		asprintf(&sql,"INSERT INTO service_flapping VALUES ('%s','%d','%d','%d','%lu','%.03f','%.03f','%.03f')", longdate, op_broker_data->type, host_id, service_id, op_broker_data->comment_id, op_broker_data->percent_change,op_broker_data->high_threshold, op_broker_data->low_threshold );
		response = db_query_exec(sql,DBNAME);
		my_free(sql);
		mysql_free_result(response);
		
		/* update service_status flapping started */
		if (op_broker_data->type==NEBTYPE_FLAPPING_START) {
			asprintf(&sql,"UPDATE service_status set is_flapping=1,flapping_comment_id='%lu',percent_state_change='%.03f' where host_id='%d' and service_id='%d'",op_broker_data->comment_id, op_broker_data->percent_change, host_id, service_id );
			response = db_query_exec(sql,DBNAME);
			my_free(sql);
			mysql_free_result(response);
			
		} else if (op_broker_data->type==NEBTYPE_FLAPPING_STOP) {
			asprintf(&sql,"UPDATE service_status set is_flapping=0,flapping_comment_id='%lu',percent_state_change='%.03f' where host_id='%d' and service_id='%d'",op_broker_data->comment_id, op_broker_data->percent_change, host_id, service_id );
			response = db_query_exec(sql,DBNAME);
			my_free(sql);
			mysql_free_result(response);
		}
		
	} else if (op_broker_data->flapping_type==HOST_FLAPPING) {
		
		host_id = opdb_find_host(op_broker_data->host_name);
		//printf("opdb_handle_flapping_data Trying to find %s\n",op_broker_data->host_name);
		if (host_id == ERROR)
			return ERROR;

		asprintf(&sql,"INSERT INTO host_flapping VALUES ('%s','%d','%d','%lu','%.03f','%.03f','%.03f')", longdate, op_broker_data->type, host_id,op_broker_data->comment_id,op_broker_data->percent_change, op_broker_data->high_threshold,op_broker_data->low_threshold);
		
		response = db_query_exec(sql,DBNAME);
		my_free(sql);
		mysql_free_result(response);
		
		/* update host_status flapping started */
		if (op_broker_data->type==NEBTYPE_FLAPPING_START) {
			asprintf(&sql,"UPDATE host_status set is_flapping=1,flapping_comment_id='%lu',percent_state_change='%.03f' where host_id='%d'",op_broker_data->comment_id, op_broker_data->percent_change, host_id);
			
			response = db_query_exec(sql,DBNAME);
			my_free(sql);
			mysql_free_result(response);
			
		} else if (op_broker_data->type==NEBTYPE_FLAPPING_STOP) {
			asprintf(&sql,"UPDATE host_status set is_flapping=0,flapping_comment_id='%lu',percent_state_change='%.03f' where host_id='%d'",op_broker_data->comment_id, op_broker_data->percent_change, host_id);
			response = db_query_exec(sql,DBNAME);
			my_free(sql);
			mysql_free_result(response);
		}
	}

	return OK;
}

/*
 * handles host and service state changes
 */
int opdb_handle_state_change(int event_type, void *data)
{
	nebstruct_statechange_data *op_broker_data=NULL;
	char *sql=NULL;
	char longdate[32];
	int host_id;
	int service_id;
	MYSQL_RES *response = NULL;
	
	op_broker_data=(nebstruct_statechange_data *)data;
	if (op_broker_data->type==NEBTYPE_STATECHANGE_END) {
		timestamp_to_longdate(longdate, op_broker_data->timestamp.tv_sec);
		remove_apostrofe(op_broker_data->output);
		
		host_id = opdb_find_host(op_broker_data->host_name);
		//printf("opdb_handle_state_change Trying to find %s\n",op_broker_data->host_name);
		if (host_id == ERROR)
			return ERROR;

		/* Checks to see which notification type, SERVICE or HOST */
		if(op_broker_data->statechange_type==SERVICE_STATECHANGE) {
			service_id = opdb_find_service(op_broker_data->host_name,
				op_broker_data->service_description);

			if (service_id == ERROR)
				return ERROR;

			asprintf(&sql,"INSERT INTO service_state_change VALUES ('%s','%d','%d',%d,%d,%d,%d,'%s')", longdate, host_id, service_id, op_broker_data->state,op_broker_data->state_type,op_broker_data->current_attempt,op_broker_data->max_attempts,op_broker_data->output);
			response = db_query_exec(sql,DBNAME);
			my_free(sql);
			mysql_free_result(response);
			
		} else if (op_broker_data->statechange_type==HOST_STATECHANGE) {
			asprintf(&sql,"INSERT INTO host_state_change VALUES ('%s','%d',%d,%d,%d,%d,'%s')", longdate, host_id, op_broker_data->state,op_broker_data->state_type,op_broker_data->current_attempt,op_broker_data->max_attempts,op_broker_data->output);
			response = db_query_exec(sql,DBNAME);
			my_free(sql);
			mysql_free_result(response);
		}
	}
	
	return OK;
}


/*
 * handle host and service comments
 */
int opdb_handle_comment(int event_type, void *data)
{

	nebstruct_comment_data *op_broker_data=NULL;
	char *sql=NULL;
	char longdate[32];
	char entry_time[32];
	char expire_time[32];
	int host_id;
	int service_id;
	MYSQL_RES *response = NULL;
	
	op_broker_data=(nebstruct_comment_data *)data;
	if (op_broker_data->type==NEBTYPE_COMMENT_DELETE) {
		if(op_broker_data->comment_type==SERVICE_COMMENT) {
			asprintf(&sql,"DELETE FROM service_comment WHERE comment_id=%lu",op_broker_data->comment_id);
		}
		else {
			asprintf(&sql,"DELETE FROM host_comment WHERE comment_id=%lu",op_broker_data->comment_id);
		}
		response = db_query_exec(sql,DBNAME);
		my_free(sql);
		mysql_free_result(response);
		
	} else if (op_broker_data->type==NEBTYPE_COMMENT_ADD) {
		
		host_id = opdb_find_host(op_broker_data->host_name);
		//printf("opdb_handle_comment Trying to find %s\n",op_broker_data->host_name);
		if (host_id == ERROR)
			return ERROR;
		
		timestamp_to_longdate(longdate, op_broker_data->timestamp.tv_sec);
		timestamp_to_longdate(entry_time, op_broker_data->entry_time);
		timestamp_to_longdate(expire_time, op_broker_data->expire_time);
		remove_apostrofe(op_broker_data->comment_data);
		
		/* Checks to see which notification type, SERVICE or HOST */
		if(op_broker_data->comment_type==SERVICE_COMMENT) {
			
			service_id = opdb_find_service(op_broker_data->host_name,
				op_broker_data->service_description);

			if (service_id == ERROR)
				return ERROR;
			
			asprintf(&sql,"INSERT INTO service_comment VALUES ('%s','%d','%d','%s','%s','%s',%d,%d,%d,%d,'%s',%lu)", longdate, host_id, service_id,entry_time,op_broker_data->author_name,op_broker_data->comment_data,op_broker_data->persistent,op_broker_data->source,op_broker_data->entry_type,op_broker_data->expires,expire_time,op_broker_data->comment_id);
			response = db_query_exec(sql,DBNAME);
			my_free(sql);
			mysql_free_result(response);
			
			/* Sets ACK flag on status table */
			if (op_broker_data->entry_type==ACKNOWLEDGEMENT_COMMENT) {
				asprintf(&sql, "UPDATE service_status SET problem_has_been_acknowledged=1 WHERE host_id='%d' AND service_id='%d'", host_id, service_id);
				response = db_query_exec(sql,DBNAME);
				my_free(sql);
				mysql_free_result(response);
			}
			
		} else if (op_broker_data->comment_type==HOST_COMMENT) {
			
			asprintf(&sql,"INSERT INTO host_comment VALUES ('%s','%d','%s','%s','%s',%d,%d,%d,%d,'%s',%lu)", longdate, host_id, entry_time,op_broker_data->author_name,op_broker_data->comment_data,op_broker_data->persistent,op_broker_data->source,op_broker_data->entry_type,op_broker_data->expires,expire_time,op_broker_data->comment_id);
			response = db_query_exec(sql,DBNAME);
			my_free(sql);
			mysql_free_result(response);
			
			/* Sets ACK flagg on status table */
			if (op_broker_data->entry_type==ACKNOWLEDGEMENT_COMMENT) {
				asprintf(&sql, "UPDATE host_status SET problem_has_been_acknowledged=1 WHERE host_id='%d'", host_id);
				response = db_query_exec(sql,DBNAME);
				my_free(sql);
				mysql_free_result(response);
			}
		}
	}
	
	return OK;
}

/*
 * handle host checks
 */
int opdb_handle_host_check(int event_type, void *data)
{
	
	nebstruct_host_check_data *op_broker_data=NULL;
	char *sql=NULL;
	host *op_host_data=NULL;
	char last_host_notification[32];
	char next_host_notification[32];
	char next_check[32];
	char last_check[32];
	char last_state_change[32];
	char last_hard_state_change[32];
	char last_time_up[32];
	char last_time_down[32];
	char last_time_unreachable[32];
	int host_id;
	MYSQL_RES *response = NULL;
	
	op_broker_data=(nebstruct_host_check_data *)data;
	
	if (op_broker_data->type==NEBTYPE_HOSTCHECK_PROCESSED) {

		host_id = opdb_find_host(op_broker_data->host_name);
		//printf("opdb_handle_host_check Trying to find %s\n",op_broker_data->host_name);
		if (host_id == ERROR)
			return ERROR;

		op_host_data=(host *)op_broker_data->object_ptr;
		remove_apostrofe(op_host_data->plugin_output);
		remove_apostrofe(op_host_data->long_plugin_output);
		remove_apostrofe(op_host_data->perf_data);
		timestamp_to_longdate(last_host_notification,
				op_host_data->last_host_notification);
		timestamp_to_longdate(next_host_notification,
				op_host_data->next_host_notification);
		timestamp_to_longdate(next_check, op_host_data->next_check);
		timestamp_to_longdate(last_check, op_host_data->last_check);
		timestamp_to_longdate(last_state_change, op_host_data->last_state_change);
		timestamp_to_longdate(last_hard_state_change,
				op_host_data->last_hard_state_change);
		timestamp_to_longdate(last_time_up, op_host_data->last_time_up);
		timestamp_to_longdate(last_time_down, op_host_data->last_time_down);
		timestamp_to_longdate(last_time_unreachable,
				op_host_data->last_time_unreachable);
		asprintf(&sql, "UPDATE host_status SET problem_has_been_acknowledged = '%d',acknowledgement_type = '%d',check_type = '%d',current_state = '%d',last_state = '%d',last_hard_state = '%d',plugin_output = '%s',long_plugin_output = '%s',perf_data = '%s',state_type = '%d',current_attempt = '%d',latency = '%.03f',execution_time = '%.03f',is_executing = '%d',check_options = '%d',notifications_enabled = '%d',last_host_notification = '%s',next_host_notification = '%s',next_check = '%s',should_be_scheduled = '%d',last_check = '%s',last_state_change = '%s',last_hard_state_change = '%s',last_time_up = '%s',last_time_down = '%s',last_time_unreachable = '%s',has_been_checked = '%d',is_being_freshened = '%d',notified_on_down = '%d',notified_on_unreachable = '%d',current_notification_number = '%d',no_more_notifications = '%d',current_notification_id = '%lu',check_flapping_recovery_notification = '%d',scheduled_downtime_depth = '%d',pending_flex_downtime = '%d',is_flapping = '%d',flapping_comment_id = '%lu',percent_state_change = '%.03f',total_services = '%d',total_service_check_interval = '%lu',modified_attributes = '%lu' where host_id='%d'", op_host_data->problem_has_been_acknowledged,op_host_data->acknowledgement_type,op_host_data->check_type,op_host_data->current_state,op_host_data->last_state,op_host_data->last_hard_state,op_host_data->plugin_output,op_host_data->long_plugin_output,op_host_data->perf_data,op_host_data->state_type,op_host_data->current_attempt,op_host_data->latency,op_host_data->execution_time,op_host_data->is_executing,op_host_data->check_options,op_host_data->notifications_enabled,last_host_notification,next_host_notification,next_check,op_host_data->should_be_scheduled,last_check,last_state_change,last_hard_state_change,last_time_up,last_time_down,last_time_unreachable,op_host_data->has_been_checked,op_host_data->is_being_freshened,op_host_data->notified_on_down,op_host_data->notified_on_unreachable,op_host_data->current_notification_number,op_host_data->no_more_notifications,op_host_data->current_notification_id,op_host_data->check_flapping_recovery_notification,op_host_data->scheduled_downtime_depth,op_host_data->pending_flex_downtime,op_host_data->is_flapping,op_host_data->flapping_comment_id,op_host_data->percent_state_change,op_host_data->total_services,op_host_data->total_service_check_interval,op_host_data->modified_attributes, host_id );
		response = db_query_exec(sql,DBNAME);
		my_free(sql);
		mysql_free_result(response);
	}
	return OK;
}


/*
 * handle service check
 */
int opdb_handle_service_check(int event_type, void *data)
{

	nebstruct_service_check_data *op_broker_data=NULL;
	service *op_service_data=NULL;
	char *sql=NULL;
	char *error_msg = NULL;
	char last_notification[32];
	char next_notification[32];
	char next_check[32];
	char last_check[32];
	char last_state_change[32];
	char last_hard_state_change[32];
	char last_time_ok[32];
	char last_time_warning[32];
	char last_time_unknown[32];
	char last_time_critical[32];
	int host_id;
	int service_id;
	int retval;
	MYSQL_RES *response = NULL;
	
	op_broker_data=(nebstruct_service_check_data *)data;

	if (op_broker_data->type==NEBTYPE_SERVICECHECK_PROCESSED) {
		

		host_id = opdb_find_host(op_broker_data->host_name);
		//printf("opdb_handle_service_check Trying to find %s\n",op_broker_data->host_name);
		if (host_id == ERROR)
			return ERROR;


		service_id = opdb_find_service(op_broker_data->host_name,
				op_broker_data->service_description);

		if (service_id == ERROR)
			return ERROR;
		
		op_service_data=(service *)op_broker_data->object_ptr;

		if (op_service_data->perf_data) {
			//opdb_process_perfdata(op_service_data->perf_data);
			retval = opdb_process_perfdata((void *)op_broker_data);
			if (retval != OK) {

				asprintf(&error_msg,"Unable to parse perfdata for %s %s: %s",
						op_broker_data->host_name, 
						op_broker_data->service_description,
						op_service_data->perf_data );

				write_to_log(error_msg,NSLOG_INFO_MESSAGE,NULL);
				my_free(error_msg);

			}
		}

		remove_apostrofe(op_service_data->plugin_output);
		remove_apostrofe(op_service_data->long_plugin_output);
		remove_apostrofe(op_service_data->perf_data);
		timestamp_to_longdate(next_check, op_service_data->next_check);
		timestamp_to_longdate(last_check, op_service_data->last_check);
		timestamp_to_longdate(last_notification, op_service_data->last_notification);
		timestamp_to_longdate(next_notification, op_service_data->next_notification);
		timestamp_to_longdate(last_state_change, op_service_data->last_state_change);
		timestamp_to_longdate(last_hard_state_change,
				op_service_data->last_hard_state_change);
		timestamp_to_longdate(last_time_ok, op_service_data->last_time_ok);
		timestamp_to_longdate(last_time_warning, op_service_data->last_time_warning);
		timestamp_to_longdate(last_time_unknown, op_service_data->last_time_unknown);
		timestamp_to_longdate(last_time_critical, op_service_data->last_time_critical);
		asprintf(&sql, "UPDATE service_status SET problem_has_been_acknowledged = '%d',acknowledgement_type = '%d',host_problem_at_last_check = '%d',check_type = '%d',current_state = '%d',last_state = '%d',last_hard_state = '%d',plugin_output = '%s',long_plugin_output = '%s',perf_data = '%s',state_type = '%d',next_check = '%s',should_be_scheduled = '%d',last_check = '%s',current_attempt = '%d',current_event_id = '%lu',last_event_id = '%lu',current_problem_id = '%lu',last_problem_id = '%lu',last_notification = '%s',next_notification = '%s',no_more_notifications = '%d',check_flapping_recovery_notification = '%d',last_state_change = '%s',last_hard_state_change = '%s',last_time_ok = '%s',last_time_warning = '%s',last_time_unknown = '%s',last_time_critical = '%s',has_been_checked = '%d',is_being_freshened = '%d',notified_on_unknown = '%d',notified_on_warning = '%d',notified_on_critical = '%d',current_notification_number = '%d',current_notification_id = '%lu',latency = '%.03f',execution_time = '%.03f',is_executing = '%d',check_options = '%d',scheduled_downtime_depth = '%d',pending_flex_downtime = '%d',is_flapping = '%d',flapping_comment_id = '%lu',percent_state_change = '%.03f',modified_attributes = '%lu' where host_id='%d' and service_id='%d'", op_service_data->problem_has_been_acknowledged,op_service_data->acknowledgement_type,op_service_data->host_problem_at_last_check,op_service_data->check_type,op_service_data->current_state,op_service_data->last_state,op_service_data->last_hard_state,op_service_data->plugin_output,op_service_data->long_plugin_output,op_service_data->perf_data,op_service_data->state_type,next_check,op_service_data->should_be_scheduled,last_check,op_service_data->current_attempt,op_service_data->current_event_id,op_service_data->last_event_id,op_service_data->current_problem_id,op_service_data->last_problem_id,last_notification,next_notification,op_service_data->no_more_notifications,op_service_data->check_flapping_recovery_notification,last_state_change,last_hard_state_change,last_time_ok,last_time_warning,last_time_unknown,last_time_critical,op_service_data->has_been_checked,op_service_data->is_being_freshened,op_service_data->notified_on_unknown,op_service_data->notified_on_warning,op_service_data->notified_on_critical,op_service_data->current_notification_number,op_service_data->current_notification_id,op_service_data->latency,op_service_data->execution_time,op_service_data->is_executing,op_service_data->check_options,op_service_data->scheduled_downtime_depth,op_service_data->pending_flex_downtime,op_service_data->is_flapping,op_service_data->flapping_comment_id,op_service_data->percent_state_change,op_service_data->modified_attributes,host_id, service_id );
		response = db_query_exec(sql,DBNAME);
		my_free(sql);
		mysql_free_result(response);
	}
	return(OK);
}

/*
 * handle hosts and services downtimes
 */
int opdb_handle_downtime(int event_type, void *data)
{

	nebstruct_downtime_data *op_broker_data=NULL;
	char *sql=NULL;
	char timestamp[32];
	char entry_time[32];
	char start_time[32];
	char end_time[32];
	int host_id;
	int service_id;
	MYSQL_RES *response = NULL;
	
	op_broker_data=(nebstruct_downtime_data *)data;
	if  ( (op_broker_data->type==NEBTYPE_DOWNTIME_ADD) ||
			(op_broker_data->type==NEBTYPE_DOWNTIME_START) ||
			(op_broker_data->type==NEBTYPE_DOWNTIME_STOP) ) {
		
		timestamp_to_longdate(timestamp, op_broker_data->timestamp.tv_sec);
		timestamp_to_longdate(entry_time, op_broker_data->entry_time);
		timestamp_to_longdate(start_time, op_broker_data->start_time);
		timestamp_to_longdate(end_time, op_broker_data->end_time);
		remove_apostrofe(op_broker_data->comment_data);
		
		host_id = opdb_find_host(op_broker_data->host_name);
		//printf("opdb_handle_downtime Trying to find %s\n",op_broker_data->host_name);
		if (host_id == ERROR)
			return ERROR;
		
		/* Host downtime */
		if (op_broker_data->service_description==NULL) {
			asprintf(&sql, "INSERT INTO host_downtime VALUES ('%d','%s','%d','%s','%s','%s','%s','%s','%d','%lu','%lu','%lu')",op_broker_data->type,timestamp,host_id,entry_time,op_broker_data->author_name,op_broker_data->comment_data,start_time,end_time,op_broker_data->fixed,op_broker_data->duration,op_broker_data->triggered_by,op_broker_data->downtime_id);
			response = db_query_exec(sql,DBNAME);
			my_free(sql);
			mysql_free_result(response);
			if (op_broker_data->type==NEBTYPE_DOWNTIME_START) {
				asprintf(&sql, "UPDATE host_status SET scheduled_downtime_depth=1 where host_id='%d'",host_id);
				response = db_query_exec(sql,DBNAME);
				my_free(sql);
				mysql_free_result(response);
			}
			
		} else { /* Service downtime */
			
			service_id = opdb_find_service(op_broker_data->host_name,
				op_broker_data->service_description);

			if (service_id == ERROR)
				return ERROR;
			
			asprintf(&sql, "INSERT INTO service_downtime VALUES ('%d','%s','%d','%d','%s','%s','%s','%s','%s','%d','%lu','%lu','%lu')",op_broker_data->type,timestamp,host_id,service_id,entry_time,op_broker_data->author_name,op_broker_data->comment_data,start_time,end_time,op_broker_data->fixed,op_broker_data->duration,op_broker_data->triggered_by,op_broker_data->downtime_id);
			response = db_query_exec(sql,DBNAME);
			my_free(sql);
			mysql_free_result(response);
			
			if (op_broker_data->type==NEBTYPE_DOWNTIME_START) {
				asprintf(&sql, "UPDATE service_status SET scheduled_downtime_depth=1 where host_id='%d' and service_id='%d'",host_id, service_id);
				response = db_query_exec(sql,DBNAME);
				my_free(sql);
				mysql_free_result(response);
			}
		}
		
	} else if (op_broker_data->type==NEBTYPE_DOWNTIME_DELETE) {
		
		host_id = opdb_find_host(op_broker_data->host_name);
		//printf("opdb_handle_downtime2 Trying to find %s\n",op_broker_data->host_name);
		if (host_id == ERROR)
			return ERROR;
		
		/* Host downtime */
		if (op_broker_data->service_description==NULL) {
			asprintf(&sql, "DELETE FROM host_downtime WHERE type='%d' AND downtime_id='%lu' AND host_id='%d'",NEBTYPE_DOWNTIME_ADD, op_broker_data->downtime_id,host_id);
			
		} else { /* Service downtime */
			
			service_id = opdb_find_service(op_broker_data->host_name,
				op_broker_data->service_description);

			if (service_id == ERROR)
				return ERROR;
			
			asprintf(&sql, "DELETE FROM service_downtime WHERE type='%d' AND downtime_id='%lu' AND host_id='%d' AND service_id='%d'",NEBTYPE_DOWNTIME_ADD,op_broker_data->downtime_id,host_id,service_id);
			
		}
		response = db_query_exec(sql,DBNAME);
		my_free(sql);
		mysql_free_result(response);
	}
	return OK;
}


/*
 * store/update service and host initial state
 */
int opdb_insert_initial_state(int event_type, void *data)
{

#ifdef DEBUG0
	printf("opdb_insert_initial_state\n");
#endif

	
	char *sql=NULL;
	host *op_host_data=NULL;
	service *op_service_data=NULL;
	char last_host_notification[32];
	char next_host_notification[32];
	char last_notification[32];
	char next_notification[32];
	char next_check[32];
	char last_check[32];
	char last_state_change[32];
	char last_hard_state_change[32];
	char last_time_up[32];
	char last_time_down[32];
	char last_time_unreachable[32];
	char last_time_ok[32];
	char last_time_warning[32];
	char last_time_unknown[32];
	char last_time_critical[32];
	int host_id;
	int service_id;
	MYSQL_RES *response = NULL;
	
	nebstruct_retention_data *op_handle = (nebstruct_retention_data *) data;
	
	if (op_handle->type != NEBTYPE_PROCESS_EVENTLOOPSTART)
		return(OK);

	/* Clear old service and host states  */
	asprintf(&sql,"delete from host_status");
	response = db_query_exec(sql,DBNAME);
	my_free(sql);
	mysql_free_result(response);
	asprintf(&sql,"delete from service_status");
	response = db_query_exec(sql,DBNAME);
	my_free(sql);
	mysql_free_result(response);
	
	/* Insert initial states for hosts */
	for( op_host_data = host_list; op_host_data != NULL;
			op_host_data = op_host_data->next ) {
		
		host_id = opdb_find_host(op_host_data->name);
		//printf("opdb_insert_initial_state Trying to find %s - Returned[%d]\n",op_host_data->name,host_id);
		if (host_id == ERROR)
			return ERROR;
		
		remove_apostrofe(op_host_data->plugin_output);
		remove_apostrofe(op_host_data->long_plugin_output);
		remove_apostrofe(op_host_data->perf_data);
		timestamp_to_longdate(last_host_notification,
			op_host_data->last_host_notification);
		timestamp_to_longdate(next_host_notification,
			op_host_data->next_host_notification);
		timestamp_to_longdate(next_check, op_host_data->next_check);
		timestamp_to_longdate(last_check, op_host_data->last_check);
		timestamp_to_longdate(last_state_change, op_host_data->last_state_change);
		timestamp_to_longdate(last_hard_state_change,
			op_host_data->last_hard_state_change);
		timestamp_to_longdate(last_time_up, op_host_data->last_time_up);
		timestamp_to_longdate(last_time_down, op_host_data->last_time_down);
		timestamp_to_longdate(last_time_unreachable,
			op_host_data->last_time_unreachable);
		asprintf(&sql, "INSERT INTO host_status( host_id,problem_has_been_acknowledged,acknowledgement_type,check_type,current_state,last_state,last_hard_state,plugin_output,long_plugin_output,perf_data,state_type,current_attempt,latency,execution_time,is_executing,check_options,notifications_enabled,last_host_notification,next_host_notification,next_check,should_be_scheduled,last_check,last_state_change,last_hard_state_change,last_time_up,last_time_down,last_time_unreachable,has_been_checked,is_being_freshened,notified_on_down,notified_on_unreachable,current_notification_number,no_more_notifications,current_notification_id,check_flapping_recovery_notification,scheduled_downtime_depth,pending_flex_downtime,is_flapping,flapping_comment_id,percent_state_change,total_services,total_service_check_interval,modified_attributes ) VALUES( '%d','%d','%d','%d','%d','%d','%d','%s','%s','%s','%d','%d','%.03f','%.03f','%d','%d','%d','%s','%s','%s','%d','%s','%s','%s','%s','%s','%s','%d','%d','%d','%d','%d','%d','%lu','%d','%d','%d','%d','%lu','%.03f','%d','%lu','%lu' )", host_id,op_host_data->problem_has_been_acknowledged,op_host_data->acknowledgement_type,op_host_data->check_type,op_host_data->current_state,op_host_data->last_state,op_host_data->last_hard_state,op_host_data->plugin_output,op_host_data->long_plugin_output,op_host_data->perf_data,op_host_data->state_type,op_host_data->current_attempt,op_host_data->latency,op_host_data->execution_time,op_host_data->is_executing,op_host_data->check_options,op_host_data->notifications_enabled,last_host_notification,next_host_notification,next_check,op_host_data->should_be_scheduled,last_check,last_state_change,last_hard_state_change,last_time_up,last_time_down,last_time_unreachable,op_host_data->has_been_checked,op_host_data->is_being_freshened,op_host_data->notified_on_down,op_host_data->notified_on_unreachable,op_host_data->current_notification_number,op_host_data->no_more_notifications,op_host_data->current_notification_id,op_host_data->check_flapping_recovery_notification,op_host_data->scheduled_downtime_depth,op_host_data->pending_flex_downtime,op_host_data->is_flapping,op_host_data->flapping_comment_id,op_host_data->percent_state_change,op_host_data->total_services,op_host_data->total_service_check_interval,op_host_data->modified_attributes );
		response = db_query_exec(sql,DBNAME);
		my_free(sql);
		mysql_free_result(response);
	}
	
	/* Insert initial states for services */
	for( op_service_data = service_list; op_service_data != NULL;
				op_service_data = op_service_data->next ) {
		
		host_id = opdb_find_host(op_service_data->host_name);
		//printf("opdb_insert_initial_state2 Trying to find %s\n",op_service_data->host_name);
		if (host_id == ERROR)
			return ERROR;
		service_id = opdb_find_service(op_service_data->host_name,
				op_service_data->description);

		if (service_id == ERROR)
			return ERROR;
		
		remove_apostrofe(op_service_data->plugin_output);
		remove_apostrofe(op_service_data->long_plugin_output);
		remove_apostrofe(op_service_data->perf_data);
		timestamp_to_longdate(next_check, op_service_data->next_check);
		timestamp_to_longdate(last_check, op_service_data->last_check);
		timestamp_to_longdate(last_notification, op_service_data->last_notification);
		timestamp_to_longdate(next_notification, op_service_data->next_notification);
		timestamp_to_longdate(last_state_change, op_service_data->last_state_change);
		timestamp_to_longdate(last_hard_state_change,
				op_service_data->last_hard_state_change);
		timestamp_to_longdate(last_time_ok, op_service_data->last_time_ok);
		timestamp_to_longdate(last_time_warning, op_service_data->last_time_warning);
		timestamp_to_longdate(last_time_unknown, op_service_data->last_time_unknown);
		timestamp_to_longdate(last_time_critical, op_service_data->last_time_critical);
		asprintf(&sql, "INSERT INTO service_status( host_id,service_id,problem_has_been_acknowledged,acknowledgement_type,host_problem_at_last_check,check_type,current_state,last_state,last_hard_state,plugin_output,long_plugin_output,perf_data,state_type,next_check,should_be_scheduled,last_check,current_attempt,current_event_id,last_event_id,current_problem_id,last_problem_id,last_notification,next_notification,no_more_notifications,check_flapping_recovery_notification,last_state_change,last_hard_state_change,last_time_ok,last_time_warning,last_time_unknown,last_time_critical,has_been_checked,is_being_freshened,notified_on_unknown,notified_on_warning,notified_on_critical,current_notification_number,current_notification_id,latency,execution_time,is_executing,check_options,scheduled_downtime_depth,pending_flex_downtime,is_flapping,flapping_comment_id,percent_state_change,modified_attributes ) VALUES( '%d','%d','%d','%d','%d','%d','%d','%d','%d','%s','%s','%s','%d','%s','%d','%s','%d','%lu','%lu','%lu','%lu','%s','%s','%d','%d','%s','%s','%s','%s','%s','%s','%d','%d','%d','%d','%d','%d','%lu','%.03f','%.03f','%d','%d','%d','%d','%d','%lu','%.03f','%lu' )", host_id,service_id,op_service_data->problem_has_been_acknowledged,op_service_data->acknowledgement_type,op_service_data->host_problem_at_last_check,op_service_data->check_type,op_service_data->current_state,op_service_data->last_state,op_service_data->last_hard_state,op_service_data->plugin_output,op_service_data->long_plugin_output,op_service_data->perf_data,op_service_data->state_type,next_check,op_service_data->should_be_scheduled,last_check,op_service_data->current_attempt,op_service_data->current_event_id,op_service_data->last_event_id,op_service_data->current_problem_id,op_service_data->last_problem_id,last_notification,next_notification,op_service_data->no_more_notifications,op_service_data->check_flapping_recovery_notification,last_state_change,last_hard_state_change,last_time_ok,last_time_warning,last_time_unknown,last_time_critical,op_service_data->has_been_checked,op_service_data->is_being_freshened,op_service_data->notified_on_unknown,op_service_data->notified_on_warning,op_service_data->notified_on_critical,op_service_data->current_notification_number,op_service_data->current_notification_id,op_service_data->latency,op_service_data->execution_time,op_service_data->is_executing,op_service_data->check_options,op_service_data->scheduled_downtime_depth,op_service_data->pending_flex_downtime,op_service_data->is_flapping,op_service_data->flapping_comment_id,op_service_data->percent_state_change,op_service_data->modified_attributes );
		response = db_query_exec(sql,DBNAME);
		my_free(sql);
		mysql_free_result(response);
	}
	
	/* Unload this callback to free unused allocated memory */
	neb_deregister_callback(NEBCALLBACK_PROCESS_DATA,opdb_insert_initial_state); 
	
	return OK;
}


/*
 * Populate the hosts linked list
 */
int opdb_populate_host_list()
{

	MYSQL_RES *response = NULL;
	MYSQL_ROW line;
	char query[] = "select host_id,host_name,use_template_id from nagios_hosts;";
	struct host_node *lf = NULL;

	response = db_query_exec(query,DBOPCFGNAME);

	if (!response)
		return ERROR;

	while ((line = mysql_fetch_row(response)) != NULL) {

		if (host_root != NULL) {
			lf = malloc(sizeof(struct host_node));
			lf->id = atoi(line[0]);
			asprintf(&lf->host_name,line[1]);
			lf->template_id = (line[2] == NULL) ? 0 : atoi(line[2]);
			lf->next = NULL;
			opdb_add_host_node(lf);
		} else {
			host_root = malloc(sizeof(struct host_node));
			host_root->id = atoi(line[0]);
			asprintf(&host_root->host_name,line[1]);
			host_root->template_id = (line[2] == NULL) ? 0 : atoi(line[2]);
			host_root->next = NULL;
		}
	}

	mysql_free_result(response);
	return OK;

}

/*
 * populate the services linked list
 */
int opdb_populate_service_list()
{

	struct host_node *pos;
	MYSQL_RES *response = NULL;
	MYSQL_ROW line;
	char *query;
	int template_id;
	struct service_node *lf = NULL;

	pos = host_root;
	while(pos != NULL) {
		template_id = pos->template_id;

		asprintf(&query,"select service_id,service_description from nagios_services where host_id='%d';",pos->id);
		response = db_query_exec(query,DBOPCFGNAME);
		free(query);
		if (!response)
			return ERROR;

		while((line = mysql_fetch_row(response)) != NULL) {
			if (service_root != NULL) {
				lf = malloc(sizeof(struct service_node));
				lf->id = atoi(line[0]);
				asprintf(&lf->host_name,pos->host_name);
				asprintf(&lf->service_description,line[1]);
				lf->next = NULL;
				opdb_add_service_node(lf);
			} else {
				service_root = malloc(sizeof(struct service_node));
				service_root->id = atoi(line[0]);
				asprintf(&service_root->host_name,pos->host_name);
				asprintf(&service_root->service_description,line[1]);
				service_root->next = NULL;
			}
		}
		mysql_free_result(response);

		/* process templates */
		while( template_id != 0) {
			asprintf(&query,"select service_id,service_description from nagios_services where host_template_id='%d';",template_id);
			response = db_query_exec(query,DBOPCFGNAME);
			free(query);
			while((line = mysql_fetch_row(response)) != NULL) {
				if (service_root != NULL) {
					lf = malloc(sizeof(struct service_node));
					lf->id = atoi(line[0]);
					asprintf(&lf->host_name,pos->host_name);
					asprintf(&lf->service_description,line[1]);
					lf->next = NULL;
					opdb_add_service_node(lf);
				} else {
					service_root = malloc(sizeof(struct service_node));
					service_root->id = atoi(line[0]);
					asprintf(&service_root->host_name,pos->host_name);
					asprintf(&service_root->service_description,line[1]);
					service_root->next = NULL;
				}
			}

			mysql_free_result(response);
			template_id = get_template_id(template_id);
		}

		pos = pos->next;

	}

	return OK;
}

/*
 * Returns the id for a given host name and service description
 * pair
 */
int opdb_find_service(char *host_name, char *service_description)
{

	struct service_node *pos;
	char *err;

#ifdef DEBUG0
	printf("find id for %s %s ->> ",host_name,service_description);
#endif

	pos = service_root;

	if (service_root == NULL)
		return ERROR;

	while(pos != NULL) {
		if (strcmp(pos->host_name,host_name) == 0 && 
			strcmp(pos->service_description,service_description) == 0) {
#ifdef DEBUG0
			printf("[%d]\n",pos->id);
#endif
			return pos->id;
		}
		pos = pos->next;
	}

	asprintf(&err,"Unable to find id for service %s in host %s",service_description,host_name);
	write_to_log(err,NSLOG_INFO_MESSAGE,NULL);
	my_free(err);
#ifdef DEBUG0
	printf("[%d]\n",ERROR);
#endif
	return ERROR;
}

/*
 * Returns the id for a given host name 
 */
int opdb_find_host(char *host_name)
{
	
	struct host_node *pos;
	char *err;


	//FILE *fp;
	//fp = fopen("/tmp/detailed.log","a");
	//fprintf(fp,"opdb_find_host Trying to find host %s\n",host_name);

	pos = host_root;
	
	if (host_root == NULL)
		return ERROR;
	
	while(pos != NULL) {
		
		//fprintf(fp,"%s == %s\n",host_name,pos->host_name);
		if (strcmp(pos->host_name,host_name) == 0 ) { 
			//fprintf(fp,"[SIM], retornando %d\n",pos->id);
			return pos->id;
			//fclose(fp);
		}
		pos = pos->next;
	}
	
	//fclose(fp);

	asprintf(&err,"Unable to find id for host %s",host_name);
	write_to_log(err,NSLOG_INFO_MESSAGE,NULL);
	my_free(err);
	return ERROR;
}


/*
 * This function add a service node at the end
 * of linked list
 */
int opdb_add_service_node(struct service_node *lf)
{
	
	struct service_node *pos;
	pos = service_root;
	while(pos->next != NULL) {
		pos = pos->next;
	}
	pos->next = lf;
	return OK;

}


/*
 * This function add a host node at the end
 * of linked list
 */
int opdb_add_host_node(struct host_node *lf)
{
	
	struct host_node *pos;
	pos = host_root;
	while(pos->next != NULL) {
		pos = pos->next;
	}
	pos->next = lf;
	return OK;
}

/*
 * convert timestamp to string in format YYYY-MM-DD HH:MM:SS
 */
void timestamp_to_longdate(char *longdate, time_t timestamp)
{
	struct tm *system_date;
	
	system_date=localtime(&timestamp);
	sprintf(longdate, "%d-%02d-%02d %02d:%02d:%02d", system_date->tm_year+1900,
		system_date->tm_mon+1, system_date->tm_mday, system_date->tm_hour,
		system_date->tm_min, system_date->tm_sec);
}

/*
 * removes caracter ' from string
 */
void remove_apostrofe(char *string)
{
	int i;
	
	if(string==NULL) {
		return;
	}
	
	for(i=0;i<strlen(string);i++) {
		if (string[i]=='\'') {
			string[i]=' ';
		}
	}
}

/*
 * connect to the database
 */
int db_connect()
{

	char *temp=NULL;
	int dbport;
	
	/* Initializes MySQL */
	dbHandle=mysql_init(NULL);
	if (!dbHandle) {
		asprintf(&temp, "MYSQL ERROR: mysql_init;0;Could not allocated memory");
		write_to_log(temp,NSLOG_INFO_MESSAGE,NULL);
		my_free(temp);
		return ERROR;
	}
	
	dbport=atoi(DBPORT);
	
	/* Connects to MySQL */
	mysql_real_connect(dbHandle, DBHOST, DBUSER, DBPASS, DBNAME,dbport,NULL,0);
	if(mysql_errno(dbHandle)) {
		asprintf(&temp, "MYSQL ERROR: mysql_real_connect;%u;%s", mysql_errno(dbHandle),
				mysql_error(dbHandle));
		write_to_log(temp,NSLOG_INFO_MESSAGE,NULL);
		my_free(temp);
		mysql_close(dbHandle);
		return ERROR;
	}

	return OK;
}

/*
 * disconnect from the database
 */
int db_disconnect()
{
	mysql_close(dbHandle);
	return OK;
}

/*
 * execute queries in the db
 */
MYSQL_RES * db_query_exec(char *sql, char *dbname)
{
	char *temp=NULL;
	int retval;
	MYSQL_RES *ret = NULL;
	
	if (mysql_ping(dbHandle)) {
		db_disconnect();
		db_connect();
		if (mysql_ping(dbHandle)) {
			asprintf(&temp, "MYSQL ERROR: mysql_ping;%u;%s;%s",
					mysql_errno(dbHandle), mysql_error(dbHandle),sql);
			write_to_log(temp,NSLOG_INFO_MESSAGE,NULL);
			my_free(temp);
			mysql_close(dbHandle);
			if (strcmp(dbname,DBNAME) == 0) {
				write_failed_query(sql, FAILED_QUERIES_FILE);
			} else {
				write_failed_query(sql, PERF_FAILED_QUERIES_FILE);
			}
			return ret;
		}
	}
	
#ifdef DEBUG0
	printf("SQL: <%s>\n", sql);
#endif

	retval = mysql_select_db(dbHandle, dbname);
	if (retval != 0) {
		asprintf(&temp, "MYSQL ERROR: Unable to select database %s;%u;%s;%s",
				dbname, mysql_errno(dbHandle), mysql_error(dbHandle),sql);
		write_to_log(temp,NSLOG_INFO_MESSAGE,NULL);
                my_free(temp);
		mysql_close(dbHandle);
		if (strcmp(dbname,DBNAME) == 0) {
			write_failed_query(sql, FAILED_QUERIES_FILE);
		} else {
			write_failed_query(sql, PERF_FAILED_QUERIES_FILE);
		}
		return ret;
	}
	
	if (mysql_query(dbHandle, sql)) {
#ifdef DEBUG0
		printf("%s\n", sql);
#endif
		/* error 1062 is insert duplicate */
		if (mysql_errno(dbHandle)!=1062) { 
			asprintf(&temp, "MYSQL ERROR: mysql_query;%u;%s;%s",
				mysql_errno(dbHandle), mysql_error(dbHandle),sql);
			write_to_log(temp,NSLOG_INFO_MESSAGE,NULL);
			my_free(temp);
			if (strcmp(dbname,DBNAME) == 0) {
				write_failed_query(sql, FAILED_QUERIES_FILE);
			} else {
				write_failed_query(sql, PERF_FAILED_QUERIES_FILE);
			}
			return ret;
		}
		
		ret = mysql_store_result(dbHandle);
		return ret;
	}

	ret = mysql_store_result(dbHandle);
	return ret;
}

/*
 * write failed query to query log file
 */
void write_failed_query(char *sql, char *file)
{
	FILE *fp;
	
	fp=fopen(file,"a+");
	if (fp) {
		fputs(sql, fp);
		fputs("\n",fp);
		fclose(fp);
	}
	
}

/*
 * parse db.php configuration
 */
void parse_db_conf()
{
	char *stringSearchDBHOST;
	char *stringSearchDBUSER;
	char *stringSearchDBPASS;
	char *stringSearchDBPORT;
	FILE *fp;
	char temp[64];
	int i, j, countquotes;
	
	/* retrieve the dbhost, username and password to connect to the mysql */
	fp=fopen(DBCONF, "r");
	if (fp) {
		while (fgets(temp, sizeof(temp), fp)) {
			stringSearchDBHOST=strstr(temp, "DBHOST");
			stringSearchDBUSER=strstr(temp, "DBUSER");
			stringSearchDBPASS=strstr(temp, "DBPASS");
			stringSearchDBPORT=strstr(temp, "DBPORT");
			
			if (stringSearchDBHOST) {
				countquotes=0;
				j=0;
				
				for (i=0;i<strlen(stringSearchDBHOST);i++) {
					if (countquotes==1) {
						DBHOST[j]=stringSearchDBHOST[i];
						j++;
					}
					if ( (stringSearchDBHOST[i]=='"') ||
						(stringSearchDBHOST[i]=='\'') ) {
						countquotes++;
					}
				}
				
				j--; /* to remove last quote */
				DBHOST[j]=0;
			}
			
			if (stringSearchDBUSER) {
				countquotes=0;
				j=0;
				for (i=0;i<strlen(stringSearchDBUSER);i++) {
					if (countquotes==1) {
						DBUSER[j]=stringSearchDBUSER[i];
						j++;
					}
					if ( (stringSearchDBUSER[i]=='"') ||
						(stringSearchDBUSER[i]=='\'') ) {
						countquotes++;
					}
				}
				j--; /* to remove last quote */
				DBUSER[j]=0;
			}
			
			if (stringSearchDBPASS) {
				countquotes=0;
				j=0;
				for (i=0;i<strlen(stringSearchDBPASS);i++) {
					if (countquotes==1) {
						DBPASS[j]=stringSearchDBPASS[i];
						j++;
					}
					if ( (stringSearchDBPASS[i]=='"') ||
						(stringSearchDBPASS[i]=='\'') ) {
						countquotes++;
					}
				}
				j--; /* to remove last quote */
				DBPASS[j]=0;
			}
			
			if (stringSearchDBPORT) {
				countquotes=0;
				j=0;
				for (i=0;i<strlen(stringSearchDBPORT);i++) {
					if (countquotes==1) {
						DBPORT[j]=stringSearchDBPORT[i];
						j++;
					}
					if ( (stringSearchDBPORT[i]=='"') ||
						(stringSearchDBPORT[i]=='\'') ) {
						countquotes++;
					}
				}
				j--; /* to remove last quote */
				DBPORT[j]=0;
			}
		}
		fclose(fp);
		
	} else {
		printf("ERROR - Could not open %s\n", DBCONF);
		exit(3);
	}
	
}

/*
 * Prints all services linked list
 */
void print_service_ll()
{
	struct service_node *pos;
	pos = service_root;
	while(pos) {
		printf("[%d] %s (%s)\n", pos->id,pos->host_name,
			pos->service_description);
		pos = pos->next;
	}
}

/*
 * Prints all hosts linked list
 */
void print_host_ll()
{
	struct host_node *pos;
	pos = host_root;
	while(pos != NULL) {
		printf("[%d] %s \n",pos->id,pos->host_name);
		pos = pos->next;
	}
}

char * replace_backslashes(char *s)
{

	int i;
	int j = 0;
	char *new_str;

	new_str = malloc(MAX_PLUGIN_OUTPUT_LENGTH);

	for(i=0;i<strlen(s);i++) {
		if (*(s+i) == '\\') {
			*(new_str+j) = '\\';
			j++;
		} else if (*(s+i) == '\'') {
			continue;
		} else if (*(s+i) == '\"') {
			continue;
		}
		*(new_str+j) = *(s+i);
		j++;
	}
	*(new_str+j) = '\x0';

	return new_str;
}

/*
 * Parse performance data
 */
int opdb_process_perfdata(void *data)
{

	regex_t re;
	regex_t metric_re;
	regmatch_t pmatch[10];
	regmatch_t metricmatch[10];
	char *straux = NULL;
	char *first_regex = NULL;
	char *latter_regex = NULL;
	char *metric_regex = NULL;
	char *str = NULL;
	char strcopy[MAX_PLUGIN_OUTPUT_LENGTH];
	int retval,i,j;
	int values = 0;
	int metrics = 0;
	int k = 0;
	struct metric mtr;
	nebstruct_service_check_data *op_broker_data = NULL;
	service *op_service_data=NULL;

	op_broker_data=(nebstruct_service_check_data *)data;
	op_service_data=(service *)op_broker_data->object_ptr;
	str = op_service_data->perf_data;

	/*
	FILE *fp;
	fp = fopen("/tmp/perfdata.log","a");
	fprintf(fp,"%s\n",str);
	fclose(fp);
	*/

	asprintf(&first_regex,"%s","([^=]{1,})=");
	asprintf(&latter_regex,"%s"," ([^=]{1,})=");

	//printf("**********************************************************\n");

	if (regcomp(&re,first_regex,REG_EXTENDED) != 0 ) {
		printf("This was not supposed to happen...\n");
		return ERROR;
	}

	straux = str;
	while(1) {

		mtr.metric = NULL;
		mtr.value = NULL;
		mtr.unit = NULL;
		mtr.warning = NULL;
		mtr.critical = NULL;
		mtr.value_min = NULL;
		mtr.value_max = NULL;

		if (metrics > 0) {
			regfree(&re);
			if (regcomp(&re,latter_regex,REG_EXTENDED) != 0 )
				return ERROR;
		}

		retval = regexec( &re, straux, 1, pmatch, 0);
#ifdef DEBUG0
		printf("[%d] ->{%s}\n",retval,straux);
#endif
		if (retval != OK) {
			// we don't parse nothing
			if (metrics == 0)
				goto outerror;
			else
				goto out;
		}

		// jump empty first char
		if (straux[pmatch[0].rm_so] == ' ')
			pmatch[0].rm_so++;

		// copy to metric structure
		for(i=pmatch[0].rm_so; i<pmatch[0].rm_eo-1; i++) {
			strcopy[i-pmatch[0].rm_so] = straux[i];
		}
		strcopy[i] = '\x0';
		mtr.metric = replace_backslashes(strcopy);
		//asprintf(&mtr.metric,"%s",strcopy);

		// jump the pointer to end of parsed string
		straux += pmatch[0].rm_eo;

		values = 0;
		memset(strcopy, 0, MAX_PLUGIN_OUTPUT_LENGTH);
		for(i=0;i<=strlen(straux);i++) {

			// metric delimiter
			if (*(straux+i) == ';' || *(straux+i) == ' ' || *(straux+i) == '\x0') {
				values++;
				switch(values) {

				// value and unit 
				case 1:

					// get the integers
					asprintf(&metric_regex,"%s","([0-9.]{1,})");
					if (regcomp(&metric_re,metric_regex,REG_EXTENDED) != 0)
						goto outerror;

					retval = regexec(&metric_re,strcopy,1,metricmatch,0);
					// the value doesn't start with integers
					if (retval != 0) {
						regfree(&metric_re);
						goto outerror;
					}

					// concatenate integers to mtr.value
					mtr.value = (char *)malloc(metricmatch[0].rm_eo);
					k = 0;
					for(j=metricmatch[0].rm_so; j<metricmatch[0].rm_eo; j++) {
						mtr.value[k] = *(strcopy+j);
						k++;
					}
					mtr.value[k] = '\x0';

					// concatenate strings to mtr.unit
					mtr.unit = (char *)malloc(strlen(strcopy));
					k = 0;
					for(j=metricmatch[0].rm_eo; j<=strlen(strcopy); j++) {
						mtr.unit[k] = *(strcopy+j);
						k++;
					}
					mtr.unit[k] = '\x0';

					my_free(metric_regex);
					regfree(&metric_re);

					break;

				// warning
				case 2:
					asprintf(&mtr.warning,"%s",strcopy);
					break;

				// critical
				case 3:
					asprintf(&mtr.critical,"%s",strcopy);
					break;

				// min
				case 4:
					asprintf(&mtr.value_min,"%s",strcopy);
					break;

				// max
				case 5:
					asprintf(&mtr.value_max,"%s",strcopy);
					goto fin;
					break;

				default:
					goto fin;
					break;

				}

				memset(strcopy, 0, MAX_PLUGIN_OUTPUT_LENGTH);

				// unfinished performance data
				if (*(straux+i) == ' ')
                                        goto fin;

			} else {
				strncat(strcopy,straux+i,1);
			}

		}

		fin:
		opdb_store_perfdata((void *)op_broker_data, &mtr);
		my_free(mtr.metric);
		my_free(mtr.value);
		my_free(mtr.unit);
		my_free(mtr.warning);
		my_free(mtr.critical);
		my_free(mtr.value_min);
		my_free(mtr.value_max);

		metrics++;

	}

	out:
	my_free(mtr.metric);
	my_free(mtr.value);
	my_free(mtr.unit);
	my_free(mtr.warning);
	my_free(mtr.critical);
	my_free(mtr.value_min);
	my_free(mtr.value_max);
	regfree(&re);
	my_free(first_regex);
	my_free(latter_regex);
	return OK;

	outerror:
	my_free(mtr.metric);
	my_free(mtr.value);
	my_free(mtr.unit);
	my_free(mtr.warning);
	my_free(mtr.critical);
	my_free(mtr.value_min);
	my_free(mtr.value_max);
	regfree(&re);
	my_free(first_regex);
	my_free(latter_regex);
	return ERROR;

}

/*
 * Stores data from a metric struture to database
 */
int opdb_store_perfdata(void *data, struct metric *m)
{

	int host_id;
	int service_id;
	int metric_id;
	char *sql = NULL;
	MYSQL_RES *result;
	MYSQL_ROW line;
	nebstruct_service_check_data *op_broker_data = NULL;

	op_broker_data=(nebstruct_service_check_data *)data;

	host_id = opdb_find_host(op_broker_data->host_name);
	if (host_id == ERROR) 
		return ERROR;

	service_id = opdb_find_service(op_broker_data->host_name,
					op_broker_data->service_description);

	if (service_id == ERROR)
		return ERROR;	

	remove_apostrofe(m->metric);
	remove_apostrofe(m->unit);
	remove_apostrofe(m->value_max);
	remove_apostrofe(m->value_min);
	asprintf(&sql,"insert into service_metric values ('',%d,%d,'%s','%s','%s','%s')",
				host_id, service_id, m->metric, m->unit, m->value_max, m->value_min );
	result = db_query_exec(sql,DBPERFNAME);
	my_free(sql);
	mysql_free_result(result);


	// get metric id
	asprintf(&sql,"select metric_id from service_metric where host_id=%d and service_id=%d and metric='%s'",
			host_id, service_id, m->metric);

	result = db_query_exec(sql,DBPERFNAME);
	my_free(sql);
	if (!result) {
		mysql_free_result(result);
		return ERROR;
	}

	line = mysql_fetch_row(result);
	metric_id = atoi(line[0]);
	mysql_free_result(result);
	
	asprintf(&sql,"create table if not exists perf_%d ( metric_id int(11), ctime datetime, value float, warn float, critical float, primary key (metric_id,ctime), key (ctime,metric_id)) ENGINE=InnoDB DEFAULT CHARSET=latin1;",host_id);
	result = db_query_exec(sql,DBPERFNAME);
	my_free(sql);
	mysql_free_result(result);

	remove_apostrofe(m->value);
	remove_apostrofe(m->warning);
	remove_apostrofe(m->critical);
	asprintf(&sql,"insert into perf_%d values (%d, now(), '%s', '%s', '%s')",
			host_id, metric_id, m->value, m->warning, m->critical );

	result = db_query_exec(sql,DBPERFNAME);
	my_free(sql);
	mysql_free_result(result);

	/*
	FILE *fp;
	fp = fopen("/tmp/perfdata.log","a");
	fprintf(fp,"Metric to be stored:\n");
	fprintf(fp,"\tHost: %s [%d]\n",op_broker_data->host_name, host_id);
	fprintf(fp,"\tService: %s [%d]\n",op_broker_data->service_description, service_id);
	fprintf(fp,"\tMetric: %s\n",m->metric);
	fprintf(fp,"\tValue: %s\n",m->value);
	fprintf(fp,"\tUnit: %s\n",m->unit);
	fprintf(fp,"\tWarning: %s\n",m->warning);
	fprintf(fp,"\tCritical: %s\n",m->critical);
	fprintf(fp,"\tMin: %s\n",m->value_min);
	fprintf(fp,"\tMax: %s\n",m->value_max);
	fclose(fp);
	*/

	return OK;

}

/*
 * Get template of template's id
 */
int get_template_id(int template_id)
{
	char *query = NULL;
	MYSQL_RES *response = NULL;
	MYSQL_ROW line;
	int ret = 0;

	asprintf(&query,"select use_template_id from nagios_host_templates where host_template_id='%d';",template_id);
	response = db_query_exec(query,DBOPCFGNAME);
	if (!response)
		return ERROR;

	while((line = mysql_fetch_row(response)) != NULL) {
		ret = (line[0] == NULL) ? 0 : atoi(line[0]);
	}

	mysql_free_result(response);
	free(query);
	return ret;
}
